﻿(function(Otime){

	Otime.Comments	= new function Comments(){
//		"use strict";

		var _this	= this;

		_this.name	= "comments";

		_this.model	= "CommentsListModel";

		_this.language	= null;

		_this.module	= '';

		_this.appModel	= null;

		_this.Types	= {
			COMMENTS: 1,
			ACTIVITIES: 2,
			ATTACHMENTS: 4
		};

		_this.all_users	= {};

		/***********************************************************************
		 * Load & show all reply for an given comment id
		 * 
		 * @param task_id
		 * @returns {Function}
		 */
		var showReplies = function(module_type, module_id){
			return function(item){
				if (item.is_reply_list())
				{
					item.is_reply_list(false);
					return;
				}
				Otime.app.loadData(new Otime.ContextModel({
					url : Otime.siteUrl + "/comments/listing/" + item.comment_id(),
					data : {
						module_type : module_type,
						module_id : module_id
					},
					global : false,
					callback : function(cont){
						var resp = cont.response;
						var arrReply = [];
						var dt;
						resp.items.otvalues.setKeys(resp.items.otkeys, function(itm){
							dt = Otime.stoj(itm.created_date, true);
							arrReply.push(new _this.TimeLineContent(itm, dt, null, item));
						});
						item.reply_list(arrReply);
						item.is_reply_list(true);
					}
				}), true);
			};
		};

		/***********************************************************************
		 * Move the attachments
		 */
		var moveAttachment = function(fileName, attachmentId, clbk){
			var postData = {
				attachment_id : attachmentId,
				file_name : fileName
			};
			Otime.app.postData(new Otime.ContextModel(Otime.siteUrl + "/attachment/move_attachments", postData, function(cont){
				clbk && typeof clbk == "function" && clbk();
			}), true);
		};

		/***********************************************************************
		 * Prepend the newly posted comments to the task.timeline array.
		 */
		var prependNewComment = function(resp, self, parent){
			if (resp.items && resp.items[0])
			{
				var newData = resp.items[0];
				newData.timeline_type = "comments";
				parent.timeline.prependSingle(new _this.TimeLineContent(newData, Otime.stoj(resp.items[0].created_date, true), function(){
					if (resp.attachments && resp.attachments[0])
					{
						return resp.attachments[0];
					}
				}, parent));
			}
			self.cancel();
		};

		/***********************************************************************
		 * Prepend the newly added attachment to the task.timeline array.
		 */
		var prependNewDocument = function(resp, self, parent){
			if (resp.attachment_id)
			{
				resp.timeline_type = "attach";
				parent.timeline.prependSingle(new _this.TimeLineContent(resp, Otime.stoj(resp.posted_date, true), null, parent));
			}
			self.cancel();
		};

		/***********************************************************************
		 * New Comment Save
		 */
		var saveComment = function(attach, savedFileName, self, parent, clbk){
			var postData = {
				comment_id : 0,
				message : self.message(),
				module_id : self.module_id,
				module_type : self.module_type,
				parent_comment_id : 0,
				attachments : attach ? 1 : 0,
				attachment_details : attach || null,
				attachment_image : self.attachment_image(),
				get_item : 1
			};

			Otime.app.postData(new Otime.ContextModel(Otime.siteUrl + "/comments/save", postData, function(cont){
				var resp = cont.response;

				if (attach && savedFileName && resp)
				{
					moveAttachment(savedFileName, resp.attachment_id, function(){
						prependNewComment(resp, self, parent);
						typeof clbk === 'function' && clbk();
					});
					return true;
				}

				prependNewComment(resp, self, parent);
				typeof clbk === 'function' && clbk();
			}), true);
		};

		var saveReply = function(data){
			if (!data.message().trim())
			{
				return false;
			}
			var postData = {
				comment_id : 0,
				message : data.message(),
				module_id : data.module_id,
				module_type : data.module_type,
				parent_comment_id : data.commentItem.comment_id()
			};

			Otime.app.postData(new Otime.ContextModel(Otime.siteUrl + "/comments/save", postData, function(cont){
				var resp = cont.response;
				postData.comment_id = resp.comment_id;
				postData.create_data = resp.create_data;
				postData.created_by = Otime.userDetails.id;
				postData.timeline_type = "comments";
				data.commentItem.reply_list.push(new _this.TimeLineContent(postData, Otime.stoj(postData.create_data, true), null, data.commentItem));
				data.commentItem.is_reply_list(true);
				data.message("");
			}), true);
		};

//		_this.fileapi	= new FileAPI({
//			dropZone : "#right-pane, #bottom-pane"
//		});
//		_this.fileapi.dropZoneSelector	= "#right-pane, #bottom-pane";
//		_this.fileapi.url = Otime.siteUrl + '/attachment/image_upload';
//		_this.fileapi.buttonValue = '<span class="icon icon-paper-clip"></span>&nbsp;&nbsp;Attach a File';
//		_this.fileapi.buttonClass = "btn btn-default btn-sm";

		/***********************************************************************
		 * 
		 * @param data
		 * @param parent
		 */
		_this.NewActivityModel = function(data, parent){
			var self = this;

			data = data || {};

			self.message = Otime.o("");
			self.module_id = data.module_id;
			self.module_type = data.module_type;
			self.attachment_image = Otime.o("");

			self.show_controls = Otime.o(false);

			$("#right-pane, #bottom-pane").unbind("dragenter").unbind("dragleave").unbind("dragover").unbind("drop");
			self.fileapi = new FileAPI({
				dropZone : data.dropZone || "#timeline"// #right-pane, #bottom-pane
			});
			self.fileapi.url = Otime.siteUrl + '/attachment/image_upload';
			self.fileapi.buttonValue = '<span class="icon icon-paper-clip"></span>&nbsp;&nbsp;Attach a File';
			self.fileapi.buttonClass = "btn btn-default btn-sm";

			var computeShowControls = function(){
				if (!self.message() && !self.fileapi.fileQueue.length)
				{
					self.show_controls(false);
				}
				else
				{
					self.show_controls(true);
				}
			};
			self.message.subscribe(computeShowControls);
			self.fileapi.onAddFile = computeShowControls;
			self.fileapi.onRemoveFile = computeShowControls;

			// self.onBlur = function(){
			// computeShowControls();
			// };
			// self.onFocus = function(){
			// self.show_controls(true);
			// };

			var saveFlag = false;
			self.save = function(){
				if (saveFlag)
				{
					return false;
				}
				var queueLength = self.fileapi.fileQueue.length;

				// Post as comment with attachment.
				if (self.message().trim() != "")
				{
					if (queueLength == 1)
					{
						saveFlag = true;
						self.fileapi.onAfterUpload = function(img_data, fileObj){
							var fileSize = fileObj.file.size;
							var filename = fileObj.file.name;
							var files = filename.split('.');

							saveFlag = false;
							if (img_data[0])
							{
								saveFlag = true;
								saveComment({
									module_id : 0,
									module_type : "",
									file_type : files.pop(),
									file_name : files.join('.'),
									file_size : fileSize,
									title : ""
								}, img_data[2].file_name, self, parent, function(){
									saveFlag = false;
									parent && parent.has_comments && parent.has_comments(true);
								});
							}
						};
						self.fileapi.uploadQueue();
						return true;
					}
					else
					{
						saveFlag = true;
						saveComment(null, null, self, parent, function(){
							saveFlag = false;
							parent && parent.has_comments && parent.has_comments(true);
						});
					}
				}

				if (queueLength > 0) // Post as attachments;
				{
					saveFlag = true;
					self.fileapi.onAfterUpload = function(img_data, fileObj){
						var fileSize = fileObj.file.size;
						var filename = fileObj.file.name;
						var files = filename.split('.');

						saveFlag = false;
						if (img_data[0])
						{
							var url = Otime.siteUrl + "/tasks/store/" + self.module_id + "/document";
							if (self.module_type == "P")
							{
								url = Otime.siteUrl + "/project/store/" + self.module_id + "/document";
							}

							var postData = {
								module_id : self.module_id,
								module_type : self.module_type,
								new_attachment : [ {
									attachment_id : 0,
									file_size : fileSize,
									file_type : files.pop(),
									file_name : files.join('.'),
									module_id : self.module_id,
									module_type : self.module_type,
									posted_by : 0
								} ]
							};
							saveFlag = true;
							Otime.app.postData(new Otime.ContextModel(url, postData, function(cont){
								var resp = cont.response;
								moveAttachment(img_data[2].file_name, resp.attachment_id, function(){
									prependNewDocument(resp, self, parent);
								});
								saveFlag = false;
								parent && parent.has_attachments && parent.has_attachments(true);
							}), true);
						}
					};
					self.fileapi.uploadQueue();
				}
			};

			self.cancel = function(){
				self.message("");
				self.attachment_image("");
				self.fileapi.clearList();
				computeShowControls();
			};
		};

		_this.TimeLineContent = function(data, dt, getCommentAttachment, parent){
			var self = this;

			data = data || {};

			// Common Variable.
			self.date = Otime.o(new Date(dt));

			self.timeline_type = Otime.o(data.timeline_type);

//			self.getRoot = function(){
//				return root;
//			};

			// Comments variables
			self.first_name = Otime.o(data.first_name || "");
			self.user_image = Otime.o(Otime.USIP + (data.user_image || "noimgenc.jpg"));

			if (self.timeline_type() != 'attach')
			{
				self.message = Otime.o(data.message || "");
				self.reply_count = Otime.o((data.reply_count || 0) - 0);
				self.a_mode = Otime.o(data.a_mode || "TE");
				self.comment_id = Otime.o(data.comment_id || 0);
				self.parent_comment_id = Otime.o(data.parent_comment_id || 0);
				self.can_delete = Otime.o(false);
				self.attachment_details = Otime.o(null);
				if (data.attachments - 0)
				{
					// self.attachment_details(new
					// Otime.AttachmentDetailModel(getCommentAttachment(self.comment_id())));
					var attachData = getCommentAttachment(self.comment_id());
					attachData.timeline_type = "attach";
					self.attachment_details(new _this.TimeLineContent(attachData, dt, null, parent));
				}

				self.new_reply = Otime.o(null);
				self.is_reply_list = Otime.o();
				self.reply_list = Otime.a();

				var module_id = data.module_id;
				var module_type = data.module_type;
				if (self.parent_comment_id() - 0)
				{
					self.delete_reply = function(comment){
						return function(item, elem){
							if (!confirm("Are you sure you want to delete this reply ?"))
							{
								return;
							}
							Otime.app.postData(new Otime.ContextModel(Otime.siteUrl + "/comments/delete_reply_comment/" + item.comment_id(), {
								module_id : module_id,
								module_type : module_type
							}, function(cont){
								comment.reply_list.remove(item);
								comment.reply_count(comment.reply_list().length);
							}), true);
						}
					};
				}
				else
				{
					self.delete_comment = function(item, elem){
						if (!confirm("Are you sure you want to delete this comment ?"))
						{
							return;
						}
						Otime.app.postData(new Otime.ContextModel(Otime.siteUrl + "/comments/delete/" + item.comment_id(), {
							module_id : module_id,
							module_type : module_type
						}, function(){
							parent.timeline.remove(item);
						}), true);
					};
				}
			}

			self.created_by = Otime.o(data.created_by || "");
			if (data.created_by && Object.keys(_this.all_users).length)
			{
				var createdBy = _this.all_users[data.created_by];
				self.first_name(createdBy.first_name);
				self.user_image = Otime.o(Otime.USIP + (createdBy.user_image || "noimgenc.jpg"));

				self.can_delete && self.can_delete(self.created_by() == Otime.UID);
			}

			if (data.posted_by)
			{
				self.first_name(data.posted_by)
			}

			if (self.timeline_type() == "attach")
			{
				self.attachment_id = Otime.o(data.attachment_id || 0);
				self.encrypt_key = Otime.o(data.encrypt_key || "");
				self.file_name = Otime.o(data.file_name || "");
				self.file_path = Otime.o(data.file_path || "");
				self.file_type = Otime.o(data.file_type || "");
				self.file_size = Otime.o(data.file_size || "");
				self.posted_by = Otime.o(data.posted_by || "");
				self.encrypted_url = Otime.o();
				self.download_url = Otime.o();
				self.tooltip = Otime.o();

				var tooltipContent = "";
				tooltipContent += self.file_name() + "." + self.file_type() + "\n";
				tooltipContent += "File Size :\t\t\t" + bytesToSize(self.file_size()) + "\n";
				tooltipContent += "Posted by :\t\t" + self.posted_by() + "\n";

				self.tooltip(tooltipContent);

				self.isNoImg = Otime.o(false);
				if (([ "jpg", "jpeg", "png", "svg", "gif" ]).indexOf(self.file_type().toLowerCase()) < 0)
				{
					self.isNoImg(true);
				}

				self.encrypted_url(Otime.baseUrl + "index.php/attachment/file/" + self.encrypt_key());
				self.download_url(Otime.baseUrl + "index.php/attachment/download/" + self.encrypt_key());
			}
		};

		_this.loadTimeLineData = function(objModule, params, clbk){
			var module_type	= params.module_type;
			var module_id	= params.module_id;
			var parent	= params.parent;

			objModule.newActivity(new _this.NewActivityModel({
				module_type : module_type,
				module_id : module_id,
				dropZone: params.dropZone || null
			}, objModule, parent));
			if (objModule.timeline() != null && objModule.timeline.locked)
			{
				return;
			}
			Otime.app.loadData(new Otime.ContextModel({
				url : Otime.siteUrl + "/comments/timeline",
				data : {
					filter: params.filter || "all",
					module_type : module_type,
					module_id : module_id
				},
				global : false,
				callback : function(cont){
					var response = cont.response;
					var ret = {};
					var time, keydate, keytime;

					var dates = [], times = [];

					var fillData = function(date, val){
						time = Otime.stoj(date, true);
						keydate = time.format("YYYY-MM-DD");
						keytime = time.format("OO-NN-SS");

						if (!ret[keydate])
							ret[keydate] = {};
						if (!ret[keydate][keytime])
							ret[keydate][keytime] = [];

						if (dates.indexOf(keydate) < 0)
							dates.push(keydate);
						if (times.indexOf(time.getTime()) < 0)
							times.push(time.getTime());

						ret[keydate][keytime].push(val);
					};

					if (response.all_users && Object.keys(response.all_users).length > 0)
					{
						_this.all_users = response.all_users.otvalues.setKeys(response.all_users.otkeys, null, "user_id");
					}

					if (response.attachment && Object.keys(response.attachment).length > 0)
					{
						response.attachment.otvalues.setKeys(response.attachment.otkeys, function(itm){
							itm.timeline_type = "attach";
							fillData(itm.posted_date, itm);
						});
					}
					if (response.activity && Object.keys(response.activity).length > 0)
					{
						response.activity.otvalues.setKeys(response.activity.otkeys, function(itm){
							itm.timeline_type = "activity";
							fillData(itm.created_date, itm);
						});
					}
					if (response.comments && Object.keys(response.comments).length > 0)
					{
						response.comments.otvalues.setKeys(response.comments.otkeys, function(itm){
							itm.timeline_type = "comments";
							fillData(itm.created_date, itm);
						});
					}
					if (response.comments_attachments && Object.keys(response.comments_attachments).length > 0)
					{
						response.comments_attachments = response.comments_attachments.otvalues.setKeys(response.comments_attachments.otkeys, null, "module_id");
					}

					dates.mergeSort().reverse();
					times.mergeSort().reverse();

					var getCommentAttachment = function(commentId){
						return response.comments_attachments[commentId];
					};

					objModule.show_replies = showReplies(module_type, module_id);
					objModule.reply = function(commentItem){
						if (commentItem.new_reply())
						{
							commentItem.new_reply(null);
						}
						else
						{
							commentItem.new_reply({
								message : Otime.o(),
								onKeyPress : function(itm, e){
									if (e.keyCode == 13)
									{
										saveReply({
											message: this.message,
											commentItem: commentItem,
											module_type: module_type,
											module_id: module_id
										});
										return false;
									}
									return true;
								}
							});
						}
					};

					var arrTimeLine = [];
					for (var i = 0, iLen = times.length, dt = null, timeList = []; i < iLen; i++)
					{
						dt = new Date(times[i]);
						timeList = ret[dt.format("YYYY-MM-DD")][dt.format("OO-NN-SS")];
						for (var j = 0, jLen = timeList.length; j < jLen; j++)
						{
							arrTimeLine.push(new _this.TimeLineContent(timeList[j], dt, getCommentAttachment, objModule, parent));
						}
					}
					objModule.timeline(arrTimeLine);
					objModule.timeline.locked = true;
					setTimeout(function(){
						objModule.timeline.locked = false;
					}, 300000);
					typeof clbk === 'function' && clbk(objModule, cont);
				}
			}), true);
		};

		_this.init	= function(params){
			var self	= this;

			Otime.unbind();

			_this.edit_container	= $('#add-new-container');

			self.container	= params.elem || null;

			self.model	= params.model || new function(){
				var _self	= this;

				_self.newActivity	= params.newActivity || Otime.o();
				_self.timeline	= params.timeline || Otime.a();
				_self.filter	= Otime.o("all");
				_self.is_comment	= Otime.o(true);
				_self.is_document	= Otime.o(true);
				_self.is_activity	= Otime.o(true);
				var types	= {
					attachments: function(){
						_self.filter("attachments");
						_self.is_comment(false);
						_self.is_document(true);
						_self.is_activity(false);
					},
					comments: function(){
						_self.filter("comments");
						_self.is_comment(true);
						_self.is_document(false);
						_self.is_activity(false);
					},
					activity: function(){
						_self.filter("activity");
						_self.is_comment(false);
						_self.is_document(false);
						_self.is_activity(true);
					},
					all: function(){
						_self.filter("all");
						_self.is_comment(true);
						_self.is_document(true);
						_self.is_activity(true);
					}
				};
				_self.loadItem	= function(filter){
					return types[filter] ? types[filter] : function(){};
				}
			};
			// new _this.TimelineModel(params);

			_this.loadTimeLineData(self.model, params, null);

			if (self.container)
			{
				ko.applyBindings(self.model, self.container.get(0));
			}

			// self.model.loadInfo();

			if (self.container)
			{
				return self;
			}

			return self.model;
		};
	};
}(window.Otime));
